﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Web.UI.WebControls;

namespace KRO
{
	public static class Extensions
	{
		/// <summary>
		/// Sorts any IEnumerable collection of any objects by any field in any direction
		/// </summary>
		/// <typeparam name="T">any object</typeparam>
		/// <param name="source">source collection</param>
		/// <param name="sortExpression">field name of sorting object</param>
		/// <param name="direction">sorting direction</param>
		/// <returns>sorted collection of objects</returns>
		public static IEnumerable<T> SortBy<T>(this IEnumerable<T> source, string sortExpression, SortDirection direction)
		{
			if (source == null)
			{
				throw new ArgumentNullException("source");
			}

			string methodName = "OrderBy";
			if (direction == SortDirection.Descending)
			{
				methodName += "Descending";
			}

			var parametherExpression = Expression.Parameter(typeof (T), String.Empty);
			var propertyExpression = Expression.PropertyOrField(parametherExpression, sortExpression);

			var sortLambda = Expression.Lambda(propertyExpression, parametherExpression);

			var methodCallExp = Expression.Call(
				typeof (Queryable),
				methodName,
				new[] {typeof (T), propertyExpression.Type},
				source.AsQueryable().Expression,
				Expression.Quote(sortLambda)
				);

			return (IEnumerable<T>) source.AsQueryable().Provider.CreateQuery(methodCallExp);
		}

		/// <summary>
		/// Contains with StringComparision option
		/// </summary>
		/// <param name="source">source string</param>
		/// <param name="toCheck">string to check</param>
		/// <param name="comparison">StringComparison option</param>
		/// <returns>true if contains</returns>
		public static bool Contains(this string source, string toCheck, StringComparison comparison)
		{
			return source.IndexOf(toCheck, comparison) >= 0;
		}

		/// <summary>
		/// Distinct by field
		/// </summary>
		/// <typeparam name="TSource">source object</typeparam>
		/// <typeparam name="TKey">field</typeparam>
		/// <param name="source">source collection</param>
		/// <param name="keySelector">selector function</param>
		/// <returns>collection with distinct fields</returns>
		public static IEnumerable<TSource> DistinctBy<TSource, TKey>(this IEnumerable<TSource> source,
			Func<TSource, TKey> keySelector)
		{
			HashSet<TKey> seenKeys = new HashSet<TKey>();
			foreach (TSource element in source)
			{
				if (seenKeys.Add(keySelector(element)))
				{
					yield return element;
				}
			}
		}
	}
}